package com.blazinggames.urcl.ui.comp;

// Java imports
import java.awt.*;

// Blazing Games imports
import com.blazinggames.urcl.ui.sprite.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * Your standard text button.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */

public class TextButton 
extends Comp
{
// *********************
// ***** CONSTANTS *****
// *********************

    public static final int STATE_DISABLED = -1;
    public static final int STATE_NORMAL = 0;
    public static final int STATE_OVER = 1;
    public static final int STATE_SELECTED = 2;
    
    public static final Color DARK_BLUE = new Color(0, 0, 128);
    public static final Color LIGHT_BLUE = new Color(12, 128, 255);


// *********************
// ***** VARIABLES *****
// *********************
    
    protected int _state;
    protected String _text;
    protected int _thickness;
    protected boolean _isOver;
    
    protected Color _disabledColor;
    protected Color _normalColor;
    protected Color _overColor;
    protected Color _selectedColor;

    protected Color _disabledTextColor;
    protected Color _normalTextColor;
    protected Color _overTextColor;
    protected Color _selectedTextColor;


// ************************
// ***** CONSTRUCTORs *****
// ************************

    public TextButton(Component c, String text)
    {
        this(c, text, 0, 0, 200, 20);
    }
    
    public TextButton(Component c, String text, Rectangle bounds)
    {
        this(c, text, bounds.x, bounds.y, bounds.width, bounds.height);
    }
    
    public TextButton(Component c, String text, int x, int y, int w, int h)
    {
        super();
        _sprite = new TextSprite(c, text, 0,0);
        _bounds.setBounds(x, y, w, h);
        _thickness = 4;
        _text = text;
        ((TextSprite)_sprite).setFormat(TextSprite.FORMAT_SHADOW);
        setDisabledColors(Color.gray, Color.lightGray);
    	setNormalColors(Color.blue, Color.black);
    	setOverColors(DARK_BLUE, Color.black);
    	setSelectedColors(LIGHT_BLUE, Color.black);
        prepareSprite();
    }


// *****************************************************
// ***** METHODS - Information getting and setting *****
// *****************************************************

    public void setText(Component c, String text)
    {
        ((TextSprite)_sprite).setText(c, text);
        _text = text;
        _changed = true;
    }
    
    public String getText()
    {
        return _text;
    }
    
    public void setFont(Component c, Font f)
    {
        ((TextSprite)_sprite).setFont(c, f);
        _changed = true;
    }
    
    public void setThickness(int n)
    {
        _thickness = n;
        _changed = true;
    }
    
    public int getThickness()
    {
        return _thickness;
    }
    
    /**
     * Sets the colors used to render the disabled version of the button.
     *
     * @param btnCol color of the button
     * @param txtCol color of the text in the button
    */
    public void setDisabledColors(Color btnCol, Color txtCol)
    {
        _disabledColor = btnCol;
        _disabledTextColor = txtCol;
        _changed = true;
    }
    
    /**
     * Sets the colors used to render the normal version of the button.
     *
     * @param btnCol color of the button
     * @param txtCol color of the text in the button
    */
    public void setNormalColors(Color btnCol, Color txtCol)
    {
        _normalColor = btnCol;
        _normalTextColor = txtCol;
        _changed = true;
    }

    /**
     * Sets the colors used to render the mouse over version of the button.
     *
     * @param btnCol color of the button
     * @param txtCol color of the text in the button
    */
    public void setOverColors(Color btnCol, Color txtCol)
    {
        _overColor = btnCol;
        _overTextColor = txtCol;
        _changed = true;
    }

    /**
     * Sets the colors used to render the selected version of the button.
     *
     * @param btnCol color of the button
     * @param txtCol color of the text in the button
    */
    public void setSelectedColors(Color btnCol, Color txtCol)
    {
        _selectedColor = btnCol;
        _selectedTextColor = txtCol;
        _changed = true;
    }

    
    public int getState()
    {
        int oldState = _state;
        
        if (_isActive == false)
        {
            _state = STATE_DISABLED;
        }
        else if (_isSelected)
        {
            _state = STATE_SELECTED;
        }
        else if (_isOver)
        {
            _state = STATE_OVER;
        }
        else
        {
            _state = STATE_NORMAL;
        }
        
        if (_state != oldState)
        {
            prepareSprite();
        }
        
        return _state;
    }

    /**
     * sets the sprite properties based on the current state
     * of this component.
    */
    protected void prepareSprite()
    {
        TextSprite sp = (TextSprite) _sprite;

        switch (_state)
        {
            case STATE_NORMAL:
                sp.setFontColor(_normalTextColor);
                sp.setHighlightColor(_normalColor.brighter());
                break;
                
            case STATE_OVER:
                sp.setFontColor(_overTextColor);
                sp.setHighlightColor(_overColor.brighter());
                break;
                
            case STATE_SELECTED:
                sp.setFontColor(_selectedTextColor);
                sp.setHighlightColor(_selectedColor.brighter());
                break;
                
            default:
                sp.setFontColor(_disabledTextColor);
                sp.setHighlightColor(_disabledColor.brighter());
                break;                
        }
        
        _sprite.setVisible(_isVisible);
        _sprite.setBounding(false);
        _sprite.setSolid(false);
        _changed = true;
    }
    
// *******************************
// ***** METHODS - From Comp *****
// *******************************

    /**
     * Called when mouse is moved. 
     *
     * @param x x coordinate of mouse pointer
     * @param y y coordinate of mouse pointer
     * @param b this is true if mouse button currently is down
     * @return true if mouse is over component.
    */
    public boolean mouse_move(int x, int y, boolean b)
    {
        _isOver = super.mouse_move(x, y, b);
        return (getState() == STATE_OVER);
    }
    
    /**
     * Draws the component.
     *
     * @param g Graphics Context to draw into
     * @param c Component class that some awt commands need
    */
    public void render(Graphics g, Component c)
    {
        _changed = false;
        if (_isVisible == false)
            return;
        
        boolean raise = true;
        int state = getState();

        switch (_state)
        {
            case STATE_NORMAL:
                g.setColor(_normalColor);
                break;                
            case STATE_OVER:
                g.setColor(_overColor);
                break;                
            case STATE_SELECTED:
                g.setColor(_selectedColor);
                raise = false;
                break;
            default:
                g.setColor(_disabledColor);
                break;                
        }
        g.fillRect(_bounds.x, _bounds.y, _bounds.width, _bounds.height);
        
        for (int cntr = 0; cntr < _thickness; ++cntr)
        {
            g.draw3DRect(_bounds.x+cntr, _bounds.y+cntr, 
                        _bounds.width-cntr-cntr, _bounds.height-cntr-cntr, raise);
        }
        
        if (_sprite != null)
        {
            _sprite.move(_bounds.x + ((_bounds.width - _sprite.getWidth())/2),
                        _bounds.y + ((_bounds.height - _sprite.getHeight())/2) );
            _sprite.render(g, c);
            _sprite.clean();
        }
    }

}
